/* ***************
// bbcode.js
// Main implementation file for BBCode  <http://jedbrown.net/mozilla/>
//
// Author: Jed Brown (mozilla@jedbrown.net)
// Contributors:
//   - Ronny Perinke
//   - Cusser
// License:
// See license.txt file from the original .xpi.
****************** */
var BBCode_Color;
var BBCode_Size;
var gPreferences;
var BBCode_enableBBCode;
var BBCode_enableXHTML;
var BBCode_enableHTML;
var BBCode_stringBundle;
var BBCode_Document = null;
//Set Defaults
getPrefBBCodeCompose().SetDefaultBoolPref("bbcode.contextmenu.bbcode", true);
getPrefBBCodeCompose().SetDefaultBoolPref("bbcode.contextmenu.xhtml", false);
getPrefBBCodeCompose().SetDefaultBoolPref("bbcode.contextmenu.html", false);
getPrefBBCodeCompose().SetDefaultCharPref("bbcode.font.color", "#000000");
getPrefBBCodeCompose().SetDefaultCharPref("bbcode.font.size", "12");


var bbcode = {
  init: function() {
    BBCode_setupDocument(document);
    bbcode.GetUserPrefs();
    try{
      var menu = document.getElementById('contentAreaContextMenu');
      menu.addEventListener('popupshowing', bbcode.showHide, false);
    }catch (e){
      //Foobar!
    }
  },
  showHide: function() {
    if(BBCode_enableBBCode){
      document.getElementById('context-bbcode').hidden = document.getElementById('context-undo').hidden;
    }else{
      document.getElementById('context-bbcode').hidden = true;
    }
    if(BBCode_enableXHTML){
      document.getElementById('context-bbcode-xhtml').hidden = document.getElementById('context-undo').hidden;
    }else{
      document.getElementById('context-bbcode-xhtml').hidden = true;
    }
    if(BBCode_enableHTML){
      document.getElementById('context-bbcode-html').hidden = document.getElementById('context-undo').hidden;
    }else{
      document.getElementById('context-bbcode-html').hidden = true;
    }
  },
  bbcode: function(myCommand, codeType) {
    var clip = Components.classes["@mozilla.org/widget/clipboard;1"].createInstance(Components.interfaces.nsIClipboard);
    if (!clip){
      return false;
    }
    
    var trans = Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);
    if (!trans){
      return false;
    }
    
      trans.addDataFlavor("text/unicode");
      clip.getData(trans,clip.kGlobalClipboard);

    
    var str = null;
    var str_clipboard=new Object();
    var strLength=new Object();
    try{
      trans.getTransferData("text/unicode",str_clipboard,strLength);
  
  
      if (str_clipboard){
        str_clipboard=str_clipboard.value.QueryInterface(Components.interfaces.nsISupportsString);
      }
      if (str_clipboard){
        pastetext=str_clipboard.data.substring(0,strLength.value / 2);
      }
    }catch (e){
      //alert("No text in the clipboard, please copy something first.");
    }
    
    var theBox = document.commandDispatcher.focusedElement;
    var oPosition = theBox.scrollTop;
    var oHeight = theBox.scrollHeight;
    
    
    //Get Selected text (if selected)
      var startPos = theBox.selectionStart;
      var endPos = theBox.selectionEnd;
      str = theBox.value.substring(startPos, endPos);
      //alert("Str: " +str);
 
      if(codeType == "bbcode"){
        bbcode.insertAtCursorSetup(myCommand, str_clipboard, str, theBox);
      }else if(codeType == "html"){
        bbcode.insertAtCursorSetupHTML(myCommand, str_clipboard, str, theBox);
      }else if(codeType == "xhtml"){
        bbcode.insertAtCursorSetupXHTML(myCommand, str_clipboard, str, theBox);
      }else if(codeType == "bbcode-smile"){
        bbcode.insertAtCursor(theBox, " " + myCommand + " ");
      }else{
        alert("Error #1");
      }
    var nHeight = theBox.scrollHeight - oHeight;
    theBox.scrollTop = oPosition + nHeight;
  },
  
    insertAtCursorSetup: function(myCommand, str_clipboard, str, theBox) {
  // Function taken from http://www.alexking.org/blog/2003/06/02/inserting-at-the-cursor-using-javascript/
  // Modified to return cursor to correct place
    switch (myCommand){
      case "quoteclip":
        bbcode.insertAtCursor(theBox,"[quote]" + str_clipboard + "[/quote]");
        break;
        
      case "quote":
        bbcode.insertAtCursor(theBox,"[quote]" + str + "[/quote]");
        break;
        
      case "img":
        bbcode.insertAtCursor(theBox,"[img]" + str_clipboard + "[/img]");
        break;
        
      case "url":
        bbcode.insertAtCursor(theBox,"[url]" + str_clipboard + "[/url]");
        break;
        
      case "urlclip":
        bbcode.insertAtCursor(theBox,"[url=" + str_clipboard + "]" + str + "[/url]");
        break;
        
      case "bold":
        bbcode.insertAtCursor(theBox,"[b]" + str + "[/b]");
        break;
        
      case "italic":
        bbcode.insertAtCursor(theBox,"[i]" + str + "[/i]");
        break;
        
      case "underline":
        bbcode.insertAtCursor(theBox,"[u]" + str + "[/u]");
        break;
        
      case "code":
        bbcode.insertAtCursor(theBox,"[code]" + str + "[/code]");
        break;
        
      case "urlcode":
        bbcode.insertAtCursor(theBox,"[code]" + str_clipboard + "[/code]");
        break;
        
      case "list":
        var listElements = " ";
        var listString = "";
        while(listElements != "" && listElements != null){ 
          var alertText = BBCode_stringBundle.getString("bbcode.list");
          listElements = prompt(alertText);
          if(listElements != "" && listElements != null ){
            listString = listString + "\n[*]" +listElements;
          }
        }
        if(listElements != null){
          bbcode.insertAtCursor(theBox,"[list]"  + listString + "[/list]");
        }
        break;  
      
      case "size":
        var re = [];
        var bbcodesize = [BBCode_Size];
        window.openDialog("chrome://bbcode/content/bbcodesize.xul", "BBCodeSize","chrome,modal,centerscreen",re,bbcodesize);
        if(re[0]){
          bbcode.insertAtCursor(theBox,"[size=" + bbcodesize[0] + "]" + str + "[/size]");
          BBCode_Size = bbcodesize[0];
        }
        break;

      case "color":
        var re = [];
        var bbcoedcolor = [BBCode_Color];
        window.openDialog("chrome://bbcode/content/bbcodecolor.xul", "BBCodeColor","chrome,modal,centerscreen",re,bbcoedcolor);
        if(re[0]){
          bbcode.insertAtCursor(theBox,"[color=" + bbcoedcolor[0] + "]" + str + "[/color]");
          BBCode_Color = bbcoedcolor[0];
        }
        break;
        
        //End BBCODE
        
      default : 
        var alertText = BBCode_stringBundle.getString("bbcode.option.err");
        alert(alertText);
        
    }
  },

  insertAtCursorSetupHTML: function(myCommand, str_clipboard, str, theBox) {
    switch (myCommand){
        
      case "img":
        bbcode.insertAtCursor(theBox,"<img src=\"" + str_clipboard + "\"/>");
        break;
        
      case "imgalt":
        var alertText = BBCode_stringBundle.getString("bbcode.img.alttext");
        bbcode.insertAtCursor(theBox,"<img src=\"" + str_clipboard + "\"" + " alt=\"" + prompt(alertText) + "\"/>");
        break;
      
      case "mail":
        bbcode.insertAtCursor(theBox,"<a href=\"mailto:" + str_clipboard + "\">" + str_clipboard + "</a>");
        break;
        
      case "mailclip":
        bbcode.insertAtCursor(theBox,"<a href=\"mailto:" + str_clipboard + "\">" + str + "</a>");
        break;
        
        
      case "url":
        bbcode.insertAtCursor(theBox,"<a href=\"\">" + str + "</a>");
        break;
        
       case "urltitle":
        var alertText = BBCode_stringBundle.getString("bbcode.url.title");
        bbcode.insertAtCursor(theBox,"<a href=\"\" title=\""+ prompt(alertText) + "\">" + str + "</a>");
        break;
        
      case "urlclip":
        bbcode.insertAtCursor(theBox,"<a href=\"" + str_clipboard + "\">" + str + "</a>");
        break;
        
      case "urlcliptitle":
        var alertText = BBCode_stringBundle.getString("bbcode.url.title");
        bbcode.insertAtCursor(theBox,"<a href=\"" + str_clipboard + "\" title=\""+ prompt(alertText) + "\">" + str + "</a>");
        break;
        
      case "bold":
        bbcode.insertAtCursor(theBox,"<b>" + str + "</b>");
        break;
        
      case "italic":
        bbcode.insertAtCursor(theBox,"<i>" + str + "</i>");
        break;
        
      case "underline":
        bbcode.insertAtCursor(theBox,"<u>" + str + "</u>");
        break;
      
      case "strike":
        bbcode.insertAtCursor(theBox,"<strike>" + str + "</strike>");
        break;
      
      case "blockquote":
        bbcode.insertAtCursor(theBox,"<blockquote>" + str + "</blockquote>");
        break;
      
      case "list":
        var listElements = " ";
        var listString = "";
        while(listElements != "" && listElements != null){ 
          var alertText = BBCode_stringBundle.getString("bbcode.list");
          listElements = prompt(alertText);
          if(listElements != "" && listElements != null ){
            listString = listString + "\n<li>" +listElements + "</li>";
          }
        }
        if(listElements != null){
          bbcode.insertAtCursor(theBox,"<ul>"  + listString + "</ul>");
        }
        break;  
        
      case "size":
        var re = [];
        var bbcodesize = [BBCode_Size];
        window.openDialog("chrome://bbcode/content/bbcodesize.xul", "BBCodeSize","chrome,modal,centerscreen",re,bbcodesize);
        if(re[0]){
          bbcode.insertAtCursor(theBox,"<font size=\"" + bbcodesize[0] + "\">" + str + "</font>");
          BBCode_Size = bbcodesize[0];
        }
        break;

      case "color":
        var re = [];
        var bbcoedcolor = [BBCode_Color];
        window.openDialog("chrome://bbcode/content/bbcodecolor.xul", "BBCodeColor","chrome,modal,centerscreen",re,bbcoedcolor);
        if(re[0]){
          bbcode.insertAtCursor(theBox,"<font color=\"" + bbcoedcolor[0] + "\">" + str + "</font>");
          BBCode_Color = bbcoedcolor[0];
        }
        break;
        
      default : 
        var alertText = BBCode_stringBundle.getString("bbcode.option.err");
        alert(alertText);
    }
  },
    insertAtCursorSetupXHTML: function(myCommand, str_clipboard, str, theBox) {
    switch (myCommand){
        
      case "img":
        bbcode.insertAtCursor(theBox,"<img src=\"" + str_clipboard + "\"/>");
        break;
        
      case "imgalt":
        var alertText = BBCode_stringBundle.getString("bbcode.img.alttext");
        bbcode.insertAtCursor(theBox,"<img src=\"" + str_clipboard + "\"" + " alt=\"" + prompt(alertText) + "\"/>");
        break;
        
      case "mail":
        bbcode.insertAtCursor(theBox,"<a href=\"mailto:" + str_clipboard + "\">" + str_clipboard + "</a>");
        break;
        
      case "mailclip":
        bbcode.insertAtCursor(theBox,"<a href=\"mailto:" + str_clipboard + "\">" + str + "</a>");
        break;
        
      case "url":
        bbcode.insertAtCursor(theBox,"<a href=\"\">" + str + "</a>");
        break;
        
       case "urltitle":
        var alertText = BBCode_stringBundle.getString("bbcode.url.title");
        bbcode.insertAtCursor(theBox,"<a href=\"\" title=\""+ prompt(alertText) + "\">" + str + "</a>");
        break;
        
      case "urlclip":
        bbcode.insertAtCursor(theBox,"<a href=\"" + str_clipboard + "\">" + str + "</a>");
        break;
        
      case "urlcliptitle":
        var alertText = BBCode_stringBundle.getString("bbcode.url.title");
        bbcode.insertAtCursor(theBox,"<a href=\"" + str_clipboard + "\" title=\""+ prompt(alertText) + "\">" + str + "</a>");
        break;
        
      case "bold":
        bbcode.insertAtCursor(theBox,"<strong>" + str + "</strong>");
        break;
        
      case "italic":
        bbcode.insertAtCursor(theBox,"<em>" + str + "</em>");
        break;
        
      case "underline":
        bbcode.insertAtCursor(theBox,"<ins>" + str + "</ins>");
        break;
      
      case "strike":
        bbcode.insertAtCursor(theBox,"<del>" + str + "</del>");
        break;
      
      case "blockquote":
        bbcode.insertAtCursor(theBox,"<blockquote>" + str + "</blockquote>");
        break;
      
      case "list":
        var listElements = " ";
        var listString = "";
        while(listElements != "" && listElements != null){ 
          var alertText = BBCode_stringBundle.getString("bbcode.list");
          listElements = prompt(alertText);
          if(listElements != "" && listElements != null ){
            listString = listString + "\n<li>" +listElements + "</li>";
          }
        }
        if(listElements != null){
          bbcode.insertAtCursor(theBox,"<ul>"  + listString + "</ul>");
        }
        break;  
        
      case "size":
        var re = [];
        var bbcodesize = [BBCode_Size];
        window.openDialog("chrome://bbcode/content/bbcodesize.xul", "BBCodeSize","chrome,modal,centerscreen",re,bbcodesize);
        if(re[0]){
          bbcode.insertAtCursor(theBox,"<span style=\"font-size: " + bbcodesize[0] + "\">" + str + "</span>");
          BBCode_Size = bbcodesize[0];
        }
        break;

      case "color":
        var re = [];
        var bbcoedcolor = [BBCode_Color];
        window.openDialog("chrome://bbcode/content/bbcodecolor.xul", "BBCodeColor","chrome,modal,centerscreen",re,bbcoedcolor);
        if(re[0]){
          bbcode.insertAtCursor(theBox,"<span style=\"color: " + bbcoedcolor[0] + "\">" + str + "</span>");
          BBCode_Color = bbcoedcolor[0];
        }
        break;
        
      default : 
        var alertText = BBCode_stringBundle.getString("bbcode.option.err");
        alert(alertText);
    }
  },
  
  insertAtCursor_old: function(myField, myValue) {
  // Function taken from http://www.alexking.org/blog/2003/06/02/inserting-at-the-cursor-using-javascript/
  // Modified to return cursor to correct place
    if (myField.selectionStart || myField.selectionStart == '0') {
      var startPos = myField.selectionStart;
      var endPos = myField.selectionEnd;
      myField.value = myField.value.substring(0, startPos)
      + myValue
      + myField.value.substring(endPos, myField.value.length);
      var cursorPos = endPos + myValue.length;
      myField.selectionStart = cursorPos;
      myField.selectionEnd = cursorPos;
    } 
    else {
      myField.value += myValue;
    }
  },
  
  //Thanks to Nick for pointing out this function
  insertAtCursor: function(theBox, aText) {
    try {
      var command = "cmd_insertText";
      var controller = document.commandDispatcher.getControllerForCommand(command);
      if (controller && controller.isCommandEnabled(command)) {
        controller = controller.QueryInterface(Components.interfaces.nsICommandController);
        var params = Components.classes["@mozilla.org/embedcomp/command-params;1"];
        params = params.createInstance(Components.interfaces.nsICommandParams);
        params.setStringValue("state_data", aText);
        controller.doCommandWithParams(command, params);
      }
    }catch (e) {
      dump("Can't do cmd_insertText! ");
      dump(e+"\n")
    }
  },
  
  OpenSettings: function() {
    var fromEM = false;
    window.openDialog('chrome://bbcode/content/prefDialog.xul', '&bbcode.settings.window;','chrome,modal,centerscreen',fromEM,null);
  },
  GetUserPrefs: function() {
    BBCode_Size = getPrefBBCodeCompose().GetCharPref("bbcode.font.size");
    BBCode_Color = getPrefBBCodeCompose().GetCharPref("bbcode.font.color");
    BBCode_enableBBCode = getPrefBBCodeCompose().GetBoolPref("bbcode.contextmenu.bbcode");
    BBCode_enableXHTML = getPrefBBCodeCompose().GetBoolPref("bbcode.contextmenu.xhtml");
    BBCode_enableHTML = getPrefBBCodeCompose().GetBoolPref("bbcode.contextmenu.html");
  }
}

//http://www.xulplanet.com/references/xpcomref/ifaces/nsIPref.html
// Definitally the best way to use prefs, set's defaults, etc.
function getPrefBBCodeCompose() {
  if (!gPreferences) {
    try {
      gPreferences = Components.classes["@mozilla.org/preferences;1"].getService();
      gPreferences = gPreferences.QueryInterface(
        Components.interfaces.nsIPref);
    } catch (e) {}
  }

  return gPreferences;
}

//May not be needed, but just incase.
function BBCode_setupDocument(aDoc)
{
  BBCode_Document = aDoc;
  BBCode_stringBundle = aDoc.getElementById("bbcstrings");
}

//Make sure bbcode loads on start-up of browser.
window.addEventListener('load', bbcode.init, false); 
